@@ -11,6 +11,10 @@ int write_partition_number_of_heads(FILE *fp, int iHeads);
 int write_partition_start_sector_number(FILE *fp);
 
 /* Writes a physical disk drive id of 0x80 (for C:) to a partition */
-int write_partition_physical_disk_drive_id(FILE *fp);
+int write_partition_physical_disk_drive_id_fat32(FILE *fp);
+#if 0
+/* This code is not yet used */
+int write_partition_physical_disk_drive_id_fat16(FILE *fp);
+#endif
 
 #endif
