@@ -1,5 +1,5 @@
 /******************************************************************
-    Copyright (C) 2009  Henrik Carlqvist
+    Copyright (C) 2009-2016 Henrik Carlqvist
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -22,24 +22,24 @@
 
 #include "fat12.h"
 #include "fat16.h"
-#include "fat16fd.h"
 #include "fat32.h"
-#include "fat32fd.h"
-#include "fat32nt.h"
-#include "fat32pe.h"
 #include "ntfs.h"
+#include "oem_id.h"
 #include "br.h"
 #include "identify.h"
 #include "nls.h"
 #include "partition_info.h"
 
-#define VERSION "2.5.1"
+#define VERSION "2.6.0"
 
 void print_help(const char *szCommand);
 void print_version(void);
 int parse_switches(int argc, char **argv, int *piBr,
 		   int *pbForce, int *pbPrintVersion,
-		   int *pbKeepLabel, int *pbWritePartitionInfo, int *piHeads);
+		   int *pbKeepLabel, int *pbWritePartitionInfo, int *piHeads,
+		   char **pszOemId,
+		   int *pbWriteWindowsDiskSignature,
+		   uint32_t *tWindowsDiskSignature);
 int isnumber(const char *szString);
 
 int main(int argc, char **argv)
@@ -51,11 +51,16 @@ int main(int argc, char **argv)
    int bKeepLabel = 1;
    int bWritePartitionInfo = 0;
    int iHeads = -1;
+   char *szOemId = NULL;
+   int bWriteWindowsDiskSignature = 0;
+   uint32_t tWindowsDiskSignature = 0x0;
    int iRet = 0;
 
    nls_init();
    if(parse_switches(argc, argv, &iBr, &bForce, &bPrintVersion,
-		     &bKeepLabel, &bWritePartitionInfo, &iHeads))
+		     &bKeepLabel, &bWritePartitionInfo, &iHeads,
+		     &szOemId,
+		     &bWriteWindowsDiskSignature, &tWindowsDiskSignature))
    {
       print_help(argv[0]);
       return 0;
@@ -66,7 +71,9 @@ int main(int argc, char **argv)
       if(argc < 3)
 	 return 0;
    }
-   fp=fopen(argv[argc-1], (iBr || bWritePartitionInfo) ? "r+b" : "rb");
+   fp=fopen(argv[argc-1],
+	    (iBr || bWritePartitionInfo || szOemId || bWriteWindowsDiskSignature) ?
+	    "r+b" : "rb");
    if(!fp)
    {
       printf(_("Unable to open %s, %s\n"), argv[argc-1], strerror(errno));
@@ -87,6 +94,22 @@ int main(int argc, char **argv)
 	 return 1;
       }
    }
+   if(szOemId && !bForce)
+   {
+      if( ! ok_to_write_oem_id(fp, argv[argc-1], 1) )
+      {
+	 fclose(fp);
+	 return 1;
+      }
+   }
+   if(bWriteWindowsDiskSignature && !bForce)
+   {
+      if( ! sanity_check(fp, argv[argc-1], MBR_ZERO, 1) )
+      {
+	 fclose(fp);
+	 return 1;
+      }
+   }
    if( bWritePartitionInfo )
    {
       if( !iBr && !bForce)
@@ -102,7 +125,7 @@ int main(int argc, char **argv)
 	 printf(_("Start sector %ld (nr of hidden sectors) successfully written to %s\n"),
 		partition_start_sector(fp),
 		argv[argc-1]);
-	 if( write_partition_physical_disk_drive_id(fp) )
+	 if( write_partition_physical_disk_drive_id_fat32(fp) )
 	 {
 	    printf(_("Physical disk drive id 0x80 (C:) successfully written to %s\n"),
 		   argv[argc-1]);
@@ -136,7 +159,7 @@ int main(int argc, char **argv)
    {
       case NO_WRITING:
       {
-	 if( ! bWritePartitionInfo )
+	 if( (!bWritePartitionInfo) && (!szOemId) && (!bWriteWindowsDiskSignature) )
 	 {
 	    diagnose(fp, argv[argc-1]);
 	 }
@@ -178,7 +201,7 @@ int main(int argc, char **argv)
 	    printf(_("Failed writing Windows 2000/XP/2003 master boot record to %s\n"),
 		   argv[argc-1]);
 	    iRet = 1;
-	 }	    
+	 }
       }
       break;
       case MBR_95B:
@@ -191,7 +214,7 @@ int main(int argc, char **argv)
 	    printf(_("Failed writing Windows 95B/98/98SE/ME master boot record to %s\n"),
 		   argv[argc-1]);
 	    iRet = 1;
-	 }	    
+	 }
       }
       break;
       case MBR_DOS:
@@ -204,7 +227,7 @@ int main(int argc, char **argv)
 	    printf(_("Failed writing DOS/Windows NT master boot record to %s\n"),
 		   argv[argc-1]);
 	    iRet = 1;
-	 }	    
+	 }
       }
       break;
       case MBR_SYSLINUX:
@@ -214,10 +237,10 @@ int main(int argc, char **argv)
 		   argv[argc-1]);
 	 else
 	 {
-	    printf(_("Failed writing syslinux master boot record to %s\n"),
+	    printf(_("Failed writing Syslinux master boot record to %s\n"),
 		   argv[argc-1]);
 	    iRet = 1;
-	 }	    
+	 }
       }
       break;
       case MBR_GPT_SYSLINUX:
@@ -227,10 +250,75 @@ int main(int argc, char **argv)
 		   argv[argc-1]);
 	 else
 	 {
-	    printf(_("Failed writing syslinux GPT master boot record to %s\n"),
+	    printf(_("Failed writing Syslinux GPT master boot record to %s\n"),
 		   argv[argc-1]);
 	    iRet = 1;
-	 }	    
+	 }
+      }
+      break;
+      case MBR_RUFUS:
+      {
+	 if(write_rufus_mbr(fp))
+	    printf(_("Rufus master boot record successfully written to %s\n"),
+		   argv[argc-1]);
+	 else
+	 {
+	    printf(_("Failed writing Rufus master boot record to %s\n"),
+		   argv[argc-1]);
+	    iRet = 1;
+	 }
+      }
+      break;
+      case MBR_REACTOS:
+      {
+	 if(write_reactos_mbr(fp))
+	    printf(_("ReactOS master boot record successfully written to %s\n"),
+		   argv[argc-1]);
+	 else
+	 {
+	    printf(_("Failed writing ReactOS master boot record to %s\n"),
+		   argv[argc-1]);
+	    iRet = 1;
+	 }
+      }
+      break;
+      case MBR_KOLIBRIOS:
+      {
+	 if(write_kolibrios_mbr(fp))
+	    printf(_("KolibriOS master boot record successfully written to %s\n"),
+		   argv[argc-1]);
+	 else
+	 {
+	    printf(_("Failed writing KolibriOS master boot record to %s\n"),
+		   argv[argc-1]);
+	    iRet = 1;
+	 }
+      }
+      break;
+      case MBR_GRUB4DOS:
+      {
+	 if(write_grub4dos_mbr(fp))
+	    printf(_("Grub4DOS master boot record successfully written to %s\n"),
+		   argv[argc-1]);
+	 else
+	 {
+	    printf(_("Failed writing Grub4DOS master boot record to %s\n"),
+		   argv[argc-1]);
+	    iRet = 1;
+	 }
+      }
+      break;
+      case MBR_GRUB2:
+      {
+	 if(write_grub2_mbr(fp))
+	    printf(_("GRUB 2 master boot record successfully written to %s\n"),
+		   argv[argc-1]);
+	 else
+	 {
+	    printf(_("Failed writing GRUB2 master boot record to %s\n"),
+		   argv[argc-1]);
+	    iRet = 1;
+	 }
       }
       break;
       case MBR_ZERO:
@@ -243,7 +331,7 @@ int main(int argc, char **argv)
 	    printf(_("Failed writing empty (zeroed) master boot record to %s\n"),
 		   argv[argc-1]);
 	    iRet = 1;
-	 }	    
+	 }
       }
       break;
       case FAT12_BR:
@@ -256,7 +344,7 @@ int main(int argc, char **argv)
 	    printf(_("Failed writing FAT12 boot record to %s\n"),
 		   argv[argc-1]);
 	    iRet = 1;
-	 }	    
+	 }
       }
       break;
       case FAT16_BR:
@@ -269,7 +357,7 @@ int main(int argc, char **argv)
 	    printf(_("Failed writing FAT16 boot record to %s\n"),
 		   argv[argc-1]);
 	    iRet = 1;
-	 }	    
+	 }
       }
       break;
       case FAT16FD_BR:
@@ -282,7 +370,20 @@ int main(int argc, char **argv)
 	    printf(_("Failed writing FAT16 FreeDOS boot record to %s\n"),
 		   argv[argc-1]);
 	    iRet = 1;
-	 }	    
+	 }
+      }
+      break;
+      case FAT16ROS_BR:
+      {
+	 if(write_fat_16_ros_br(fp, bKeepLabel))
+	    printf(_("FAT16 ReactOS boot record successfully written to %s\n"),
+		   argv[argc-1]);
+	 else
+	 {
+	    printf(_("Failed writing FAT16 ReactOS boot record to %s\n"),
+		   argv[argc-1]);
+	    iRet = 1;
+	 }
       }
       break;
       case FAT32NT_BR:
@@ -295,7 +396,7 @@ int main(int argc, char **argv)
 	    printf(_("Failed writing FAT32 NT boot record to %s\n"),
 		   argv[argc-1]);
 	    iRet = 1;
-	 }	    
+	 }
       }
       break;
       case FAT32PE_BR:
@@ -308,7 +409,7 @@ int main(int argc, char **argv)
 	    printf(_("Failed writing FAT32 PE boot record to %s\n"),
 		   argv[argc-1]);
 	    iRet = 1;
-	 }	    
+	 }
       }
       break;
       case FAT32FD_BR:
@@ -321,7 +422,34 @@ int main(int argc, char **argv)
 	    printf(_("Failed writing FAT32 FreeDOS boot record to %s\n"),
 		   argv[argc-1]);
 	    iRet = 1;
-	 }	    
+	 }
+      }
+      break;
+      case FAT32KOS_BR:
+      {
+	 if(write_fat_32_kos_br(fp, bKeepLabel))
+	    printf(
+	       _("FAT32 KolibriOS boot record successfully written to %s\n"),
+	       argv[argc-1]);
+	 else
+	 {
+	    printf(_("Failed writing FAT32 KolibriOS boot record to %s\n"),
+		   argv[argc-1]);
+	    iRet = 1;
+	 }
+      }
+      break;
+      case FAT32ROS_BR:
+      {
+	 if(write_fat_32_ros_br(fp, bKeepLabel))
+	    printf(_("FAT32 ReactOS boot record successfully written to %s\n"),
+		   argv[argc-1]);
+	 else
+	 {
+	    printf(_("Failed writing FAT32 ReactOS boot record to %s\n"),
+		   argv[argc-1]);
+	    iRet = 1;
+	 }
       }
       break;
       case FAT32_BR:
@@ -334,7 +462,7 @@ int main(int argc, char **argv)
 	    printf(_("Failed writing FAT32 DOS boot record to %s\n"),
 		   argv[argc-1]);
 	    iRet = 1;
-	 }	    
+	 }
       }
       break;
       case NTFS_BR:
@@ -347,7 +475,7 @@ int main(int argc, char **argv)
 	    printf(_("Failed writing NTFS Windows 7 boot record to %s\n"),
 		   argv[argc-1]);
 	    iRet = 1;
-	 }	    
+	 }
       }
       break;
       default:
@@ -356,6 +484,30 @@ int main(int argc, char **argv)
       }
       break;
    }
+   if(szOemId)
+   {
+      if(write_oem_id(fp, szOemId))
+	 printf(_("OEM ID '%s' successfully written to %s\n"),
+		szOemId, argv[argc-1]);
+      else
+      {
+	 printf(_("Failed writing OEM ID to %s\n"),
+		argv[argc-1]);
+	 iRet = 1;
+      }	    
+   }
+   if(bWriteWindowsDiskSignature)
+   {
+      if(write_windows_disk_signature(fp, tWindowsDiskSignature))
+	 printf(_("Windows Disk Signature 0x%08x successfully written to %s\n"),
+		tWindowsDiskSignature, argv[argc-1]);
+	 else
+	 {
+	    printf(_("Failed writing Windows Disk Signature to %s\n"),
+		   argv[argc-1]);
+	    iRet = 1;
+	 }	    
+   }
    fclose(fp);
    return iRet;
 } /* main */
@@ -364,21 +516,27 @@ void print_help(const char *szCommand)
 {
    printf(_("Usage:\n\t%s [options] [device]\nOptions:\n"), szCommand);
    printf(
-     _("    -1, --fat12     Write a FAT12 floppy boot record to device\n"));
+      _("    -1, --fat12     Write a FAT12 floppy boot record to device\n"));
    printf(
-     _("    -2, --fat32nt   Write a FAT32 partition NT boot record to device\n"));
+      _("    -2, --fat32nt   Write a FAT32 partition NT boot record to device\n"));
    printf(
-     _("    -e, --fat32pe   Write a FAT32 partition PE boot record to device\n"));
+      _("    -e, --fat32pe   Write a FAT32 partition PE boot record to device\n"));
    printf(
-     _("    -3, --fat32     Write a FAT32 partition DOS boot record to device\n"));
+      _("    -3, --fat32     Write a FAT32 partition DOS boot record to device\n"));
    printf(
-     _("    -4, --fat32free Write a FAT32 partition FreeDOS boot record to device\n"));
+      _("    -4, --fat32free Write a FAT32 partition FreeDOS boot record to device\n"));
    printf(
-     _("    -5, --fat16free Write a FAT16 partition FreeDOS boot record to device\n"));
+      _("    -5, --fat16free Write a FAT16 partition FreeDOS boot record to device\n"));
    printf(
-     _("    -6, --fat16     Write a FAT16 partition DOS boot record to device\n"));
+      _("    -6, --fat16     Write a FAT16 partition DOS boot record to device\n"));
    printf(
-     _("    -n, --ntfs      Write a NTFS partition Windows 7 boot record to device\n"));
+      _("    -n, --ntfs      Write a NTFS partition Windows 7 boot record to device\n"));
+   printf(
+      _("    -o, --fat16ros  Write a FAT16 partition ReactOS boot record to device\n"));
+   printf(
+      _("    -c, --fat32ros  Write a FAT32 partition ReactOS boot record to device\n"));
+   printf(
+      _("    -q, --fat32kos  Write a FAT32 partition KolibriOS boot record to device\n"));
    printf(
       _("    -l, --wipelabel Reset partition disk label in boot record\n"));
    printf(
@@ -387,6 +545,12 @@ void print_help(const char *szCommand)
       _("                    to boot record\n"));
    printf(
       _("    -H, --heads <n> Manually set number of heads if partition info is written\n"));
+   printf(
+      _("    -B, --bps <n>   Manually set number of bytes per sector (default 512)\n"));
+   printf(
+      _("    -O, --writeoem <s>   Write OEM ID string <s> to file system\n"));
+   printf(
+      _("    -S, --writewds <x>   Write Windows Disk Signature hexadecimal <x> to MBR\n"));
    printf(
       _("    -7, --mbr7      Write a Windows 7 MBR to device\n"));
    printf(
@@ -398,9 +562,19 @@ void print_help(const char *szCommand)
    printf(
       _("    -d, --mbrdos    Write a DOS/Windows NT MBR to device\n"));
    printf(
-      _("    -s, --mbrsyslinux    Write a syslinux MBR to device\n"));
+      _("    -s, --mbrsyslinux    Write a Syslinux MBR to device\n"));
    printf(
-      _("    -t, --mbrgptsyslinux Write a syslinux GPT MBR to device\n"));
+      _("    -t, --mbrgptsyslinux Write a Syslinux GPT MBR to device\n"));
+   printf(
+      _("    -a, --mbrreactos     Write a ReactOS MBR to device\n"));
+   printf(
+      _("    -k, --mbrkolibrios   Write a KolibriOS MBR to device\n"));
+   printf(
+      _("    -r, --mbrrufus  Write a Rufus MBR to device\n"));
+   printf(
+      _("    -g, --mbrgrub4dos    Write a Grub4Dos MBR to device\n"));
+   printf(
+      _("    -b, --mbrgrub2  Write a Grub 2 MBR to device\n"));
    printf(
       _("    -z, --mbrzero   Write an empty (zeroed) MBR to device\n"));
    printf(
@@ -410,8 +584,9 @@ void print_help(const char *szCommand)
    printf(
       _("    -v, --version   Show program version\n"));
    printf(
-_("    -w, --write     Write automatically selected boot record to device\n\n"));
-   printf(_("    Default         Inspect current boot record\n\n"));
+      _("    -w, --write     Write automatically selected boot record to device\n\n"));
+   printf(
+      _("    Default         Inspect current boot record\n\n"));
    printf(
       _("Warning: Writing the wrong kind of boot record to a device might\n"));
    printf(
@@ -422,15 +597,17 @@ void print_version(void)
 {
    printf(_("ms-sys version %s\n"), VERSION);
    printf(_("Written by Henrik Carlqvist\n\n"));
-   printf(_("Copyright (C) 2009-2015 Free Software Foundation, Inc.\n"));
+   printf(_("Copyright (C) 2009-2016 Free Software Foundation, Inc.\n"));
    printf(_("This is free software; see the source for copying conditions.  There is NO\n"));
    printf(_("warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n\n"));
 } /* print_version */
 
 int parse_switches(int argc, char **argv, int *piBr,
-		   int *pbForce, int *pbPrintVersion,
-		   int *pbKeepLabel, int *pbWritePartitionInfo,
-		   int *piHeads)
+		   int *pbForce, int *pbPrintVersion, int *pbKeepLabel,
+		   int *pbWritePartitionInfo, int *piHeads,
+		   char **pszOemId,
+		   int *pbWriteWindowsDiskSignature,
+		   uint32_t *ptWindowsDiskSignature)
 {
    int bHelp = 0;
    int i;
@@ -441,8 +618,9 @@ int parse_switches(int argc, char **argv, int *piBr,
    *pbKeepLabel = 1;
    *pbWritePartitionInfo = 0;
    *piHeads = -1;
-   
-   
+   *pbWriteWindowsDiskSignature = 0;
+   *ptWindowsDiskSignature = 0;
+
    if(argc < 2)
       return 1;
    /* Don't parse the device */
@@ -466,6 +644,12 @@ int parse_switches(int argc, char **argv, int *piBr,
 	 *piBr = FAT32_BR;
       else if( ! strcmp("--fat32free", argv[argc]))
 	 *piBr = FAT32FD_BR;
+      else if( ! strcmp("--fat32ros", argv[argc]))
+	 *piBr = FAT32ROS_BR;
+      else if( ! strcmp("--fat32kos", argv[argc]))
+	 *piBr = FAT32KOS_BR;
+      else if( ! strcmp("--fat16ros", argv[argc]))
+	 *piBr = FAT16ROS_BR;
       else if( ! strcmp("--fat16free", argv[argc]))
 	 *piBr = FAT16FD_BR;
       else if( ! strcmp("--fat16", argv[argc]))
@@ -479,9 +663,9 @@ int parse_switches(int argc, char **argv, int *piBr,
       else if( ! strcmp("--partition", argv[argc]))
 	 *pbWritePartitionInfo = 1;
       else if( ! strcmp("--mbr7", argv[argc]))
-         *piBr = MBR_WIN7;
+	 *piBr = MBR_WIN7;
       else if( ! strcmp("--mbrvista", argv[argc]))
-         *piBr = MBR_VISTA;
+	 *piBr = MBR_VISTA;
       else if( ! strcmp("--mbr", argv[argc]))
 	 *piBr = MBR_2000;
       else if( ! strcmp("--mbr95b", argv[argc]))
@@ -492,6 +676,16 @@ int parse_switches(int argc, char **argv, int *piBr,
 	 *piBr = MBR_SYSLINUX;
       else if( ! strcmp("--mbrgptsyslinux", argv[argc]))
 	 *piBr = MBR_GPT_SYSLINUX;
+      else if( ! strcmp("--mbrrufus", argv[argc]))
+	 *piBr = MBR_RUFUS;
+      else if( ! strcmp("--mbrreactos", argv[argc]))
+	 *piBr = MBR_REACTOS;
+      else if( ! strcmp("--mbrkolibrios", argv[argc]))
+	 *piBr = MBR_KOLIBRIOS;
+      else if( ! strcmp("--mbrgrub4dos", argv[argc]))
+	 *piBr = MBR_GRUB4DOS;
+      else if( ! strcmp("--mbrgrub2", argv[argc]))
+	 *piBr = MBR_GRUB2;
       else if( ! strcmp("--mbrzero", argv[argc]))
 	 *piBr = MBR_ZERO;
       else if( ! strcmp("--write", argv[argc]))
@@ -504,6 +698,9 @@ int parse_switches(int argc, char **argv, int *piBr,
 	 {
 	    switch(argv[argc][i])
 	    {
+	       case 'a':
+		  *piBr = MBR_REACTOS;
+		  break;
 	       case '1':
 		  *piBr = FAT12_BR;
 		  break;
@@ -525,6 +722,15 @@ int parse_switches(int argc, char **argv, int *piBr,
 	       case '6':
 		  *piBr = FAT16_BR;
 		  break;
+	       case 'o':
+		  *piBr = FAT16ROS_BR;
+		  break;
+	       case 'c':
+		  *piBr = FAT32ROS_BR;
+		  break;
+	       case 'q':
+		  *piBr = FAT32KOS_BR;
+		  break;
 	       case 'n':
 		  *piBr = NTFS_BR;
 		  break;
@@ -537,12 +743,12 @@ int parse_switches(int argc, char **argv, int *piBr,
 	       case 'p':
 		  *pbWritePartitionInfo = 1;
 		  break;
-               case '7':
-                  *piBr = MBR_WIN7;
-                  break;
-               case 'i':
-                  *piBr = MBR_VISTA;
-                  break;
+	       case '7':
+		  *piBr = MBR_WIN7;
+		  break;
+	       case 'i':
+		  *piBr = MBR_VISTA;
+		  break;
 	       case 'm':
 		  *piBr = MBR_2000;
 		  break;
@@ -552,12 +758,24 @@ int parse_switches(int argc, char **argv, int *piBr,
 	       case 'd':
 		  *piBr = MBR_DOS;
 		  break;
+	       case 'r':
+		  *piBr = MBR_RUFUS;
+		  break;
 	       case 's':
 		  *piBr = MBR_SYSLINUX;
 		  break;
 	       case 't':
 		  *piBr = MBR_GPT_SYSLINUX;
 		  break;
+	       case 'g':
+		  *piBr = MBR_GRUB4DOS;
+		  break;
+	       case 'b':
+		  *piBr = MBR_GRUB2;
+		  break;
+	       case 'k':
+		  *piBr = MBR_KOLIBRIOS;
+		  break;
 	       case 'z':
 		  *piBr = MBR_ZERO;
 		  break;
@@ -576,6 +794,16 @@ int parse_switches(int argc, char **argv, int *piBr,
       else if((!strcmp("--heads", argv[argc-1]) || !strcmp("-H", argv[argc-1])) &&
 	      isnumber(argv[argc]))
 	 *piHeads = atoi(argv[argc--]);
+      else if((!strcmp("--bps", argv[argc-1]) || !strcmp("-B", argv[argc-1])) &&
+	      isnumber(argv[argc]))
+	 set_bytes_per_sector(strtoul(argv[argc--], NULL, 0));
+      else if((!strcmp("--writeoem", argv[argc-1]) || !strcmp("-O", argv[argc-1])))
+	 *pszOemId = argv[argc--];
+      else if((!strcmp("--writewds", argv[argc-1]) || !strcmp("-S", argv[argc-1])))
+      {
+	 *pbWriteWindowsDiskSignature = 1;
+	 *ptWindowsDiskSignature = strtoul(argv[argc--], NULL, 16);
+      }
       else
 	 bHelp = 1;
    }
